#' Génère des plots d'évolution de la consommation
#'
#' Cette fonction prend un dataframe, applique plusieurs filtres, agrège les données
#' selon une méthode spécifiée, et génère des plots d'évolution de la consommation.
#'
#' @param df Un dataframe contenant les données de consommation.
#' @param filters Un vecteur de filtres à appliquer aux données.
#' @param aggr.by Une chaîne de caractères spécifiant la colonne par laquelle les données doivent être agrégées
#' @param aggr.meth Une chaîne de caractères spécifiant la méthode d'agrégation ("mean", "median", ou "cumsum", par défaut "mean").
#'
#' @import ggplot2
#' @import cowplot
#'
#' @return Un plot combiné pour chaque filtre spécifié
#' @export
#'
plotEvolConsu <- function(df, filters, aggr.by = NULL, aggr.meth = "mean") {
  if (is.null(aggr.by)) {
    message("Merci de renseigner quelle variable agréger")
    stop()
  }

  # Fonction pour traiter un seul filtre
  process_filter <- function(filter) {
    # Filtrage des tendances
    dfFilt <- applyFilters(df, filters = filter)

    # Extraction et formatage des données
    dfFilt <- dfFilt %>%
      tidyr::pivot_longer(
        cols = starts_with("20"),
        names_to = "Year",
        names_prefix = "",
        values_to = "Consumption"
      ) %>%
      mutate(Year = as.integer(gsub("_Moy", "", Year))) %>%
      filter(!is.na(Consumption)) %>%
      filter(!is.na(Year))

    # Agrégation des valeurs
    dfFilt <- aggregVars(dfFilt, aggr.by, aggr.meth)

    # Création du plot à afficher
    label.plot <- ifelse(aggr.meth == "mean", "Moyenne",
      ifelse(aggr.meth == "median", "Médiane", "Cumul")
    )
    ggplot(dfFilt, aes(x = factor(Year), y = .data[[aggr.meth]], fill = .data[[aggr.by]])) +
      geom_bar(stat = "identity", position = "dodge") +
      labs(
        title = paste0(label.plot, " de Consommation (Evolution : ", filter, ")"),
        x = "Année",
        y = paste0(label.plot, " de la consommation")
      ) +
      theme_minimal()
  }

  # Appliquer la fonction à chaque filtre pour générer les plots
  plots <- lapply(filters, process_filter)

  # Extraire la légende du plot avec le vecteur CAT_ABO_DET0 le plus long
  legends <- get_legend(plots[[which.max(sapply(plots, function(p) nrow(p$data)))]])

  # Supprimer les légendes des plots individuels
  plots <- lapply(plots, function(p) p + theme(legend.position = "none"))

  # Assembler les plots avec cowplot sans légende
  combPlot <- plot_grid(plotlist = plots, labels = "AUTO")

  # Ajouter la légende combinée sur le côté droit
  finalPlot <- plot_grid(combPlot, legends, ncol = 2, rel_widths = c(1, 0.2))

  return(finalPlot)
}

#' Agrège les variables d'un dataframe selon une méthode spécifiée
#'
#' @param df Un dataframe contenant les données à agréger.
#' @param aggr.by Une chaîne de caractères spécifiant la colonne par laquelle les données doivent être agrégées.
#' @param aggr.meth Une chaîne de caractères spécifiant la méthode d'agrégation ("mean", "median", ou "cumsum").
#'
#' @return Un dataframe agrégé selon la méthode spécifiée.
#' @export
#'
aggregVars <- function(df, aggr.by, aggr.meth) {
  # Vérifier si la méthode d'agrégation existe
  if (!aggr.meth %in% c("mean", "median", "cumsum")) {
    stop(paste("Méthode d'agrégation non valide:", aggr.meth))
  }

  # Agrégation selon la méthode spécifiée
  if (aggr.meth == "cumsum") {
    df <- df %>%
      group_by(Year, .data[[aggr.by]]) %>%
      arrange(Year) %>%
      mutate(cumsum = cumsum(Consumption)) %>%
      summarize(cumsum = max(cumsum, na.rm = TRUE)) %>%
      ungroup()
  } else {
    df <- df %>%
      group_by(Year, .data[[aggr.by]]) %>%
      summarize(!!aggr.meth := match.fun(aggr.meth)(Consumption, na.rm = TRUE))
  }

  return(df)
}
