#' Affichage de données cartographiées
#'
#' Cette fonction crée une carte pour la visulaisation de données spécifiques au sein de
#' polygones géographiques, en filtrant et ajoutant des informations supplémentaires.
#'
#' @param df Un data frame de type `sf` contenant les polygones géographiques et les données associées.
#' @param var Variables de `df` à visualiser.
#' @param filters Une liste de filtres à appliquer.
#' @param add Variable suplémentaire a afficher sur les ploygones.
#' @param main Un titre pour la carte.
#'
#' @return Un objet ggplot représentant la carte avec les polygones filtrés et les données visualisées.
#' @export
#'
#' @examples
#' \dontrun{
#' df <- st_read("path/to/shapefile.shp")
#' plotMapVars(df, var = "2021_moy", filters = c("Domestique", "Croissant"), add = "2021_Effectif", main = "Carte de la Consomation")
#' }
#'
plotMapVars <- function(df, var, filters, add, main) {
  # Définition du périmètre d'information à afficher ===========================
  ## Traitement des NA's
  dfNoNa <- df %>%
    dplyr::filter(!is.na(.data[[var]]))

  ## Déterminer la zone englobante des polygones retenues de dfNoNa
  boundBox <- sf::st_bbox(dfNoNa)

  ## Créer un masque à partir de boundBox
  boundPoly <- sf::st_as_sfc(boundBox)

  ## Filtrer les polygones du shapefile original qui intersectent le masque
  dfclipped <- df %>%
    dplyr::filter(sf::st_intersects(geometry, boundPoly, sparse = FALSE))

  # Traitement des polygones sélectionnés =====================================
  ## Application des filtres
  dfFiltered <- applyFilters(dfclipped, filters)

  ## Traitement des NA's
  dfclipped <- dfclipped %>%
    dplyr::filter(!is.na(.data[[var]]))

  ## Ajouts des label sur les polygonnes contenant de la donnée
  dfCentroid <- createLabel(dfFiltered, getCommunes(dfFiltered), add) %>%
    dplyr::filter(!is.na(.data[[var]]))

  # Création de la carte =======================================================
  ggplot() +
    geom_sf(data = dfclipped, fill = "white", color = "black") +
    geom_sf(data = dfFiltered, aes_string(fill = dfFiltered %>% dplyr::pull(.data[[var]])), color = "black") +
    geom_sf_text(
      data = dfCentroid,
      aes(label = label),
      color = "black", size = 3
    ) +
    scale_fill_gradient(low = "yellow", high = "red", na.value = "transparent") +
    theme_minimal() +
    theme(
      plot.margin = margin(t = 5, r = 5, b = 5, l = 5),
      panel.spacing = unit(0, "lines"),
      legend.position = "left",
      legend.key.width = unit(1, "cm"),
      legend.key.height = unit(0.5, "cm")
    ) +
    labs(title = main, fill = var)
}
