#' Applique une série de filtres à un dataframe
#'
#' Cette fonction prend un dataframe et une liste de filtres, puis applique chaque filtre
#' aux données en conservant uniquement les lignes correspondant aux catégories spécifiées.
#'
#' @param data Un dataframe contenant les données à filtrer.
#' @param filters Une liste de catégories à utiliser comme filtres.
#'
#' @return Un dataframe filtré selon les catégories spécifiées.
#' @export
#'
applyFilters <- function(data, filters) {
  for (filter in filters) {
    # Trouver la colonne qui contient les catégories spécifiées
    column_name <- names(data)[sapply(data, function(col) any(col %in% filter))]

    if (length(column_name) > 0) {
      # Appliquer le filtre
      data <- data %>%
        dplyr::filter(.data[[column_name]] %in% c(filter, NA))
    } else {
      warning("Aucune colonne correspondante trouvée pour les catégories spécifiées")
    }
  }
  return(data)
}

#' Crée des étiquettes pour un dataframe spatial
#'
#' Cette fonction prend un dataframe spatial, crée des étiquettes pour chaque élément en combinant
#' des colonnes spécifiées et des colonnes additionnelles, puis ajoute ces étiquettes au dataframe.
#'
#' @param df Un dataframe spatial contenant les données à étiqueter.
#' @param cols Une chaîne de caractères spécifiant la colonne de base pour les étiquettes.
#' @param addCols Un vecteur de chaînes de caractères spécifiant les colonnes additionnelles à inclure dans les étiquettes.
#'
#' @return Un dataframe spatial avec une nouvelle colonne 'label' contenant les étiquettes générées.
#' @export
#'
createLabel <- function(df, cols, addCols) {
  df %>%
    sf::st_centroid() %>%
    dplyr::mutate(
      label = apply(select(df, cols, all_of(addCols)), 1, function(row) {
        base_label <- row[[cols]]
        add_labels <- sapply(addCols, function(col) {
          ifelse(is.na(row[[col]]), "", paste0("Val: ", row[[col]]))
        })
        paste(c(base_label, add_labels), collapse = "\n")
      })
    )
}

#' Identifie les colonnes de communes dans un dataframe
#'
#' Cette fonction prend un dataframe et retourne les noms des colonnes contenant des informations
#' sur les communes, basées sur un pattern de recherche spécifique.
#'
#' @param df Un dataframe contenant des données diverses.
#'
#' @return Un vecteur de chaînes de caractères représentant les noms des colonnes correspondant aux communes.
#' @export
#'
#' @examples
#' communes <- getCommunes(df)
getCommunes <- function(df) {
  ComNames <- names(df)
  ComNamesP <- ComNames[sapply(ComNames, function(col) {
    grepl("(Commune|COMUNNE|comm|Comm)", col, ignore.case = TRUE)
  })]
  return(ComNamesP)
}
